<?php

namespace Drupal\cloudflare_stream\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Defines a form that configures the settings for Cloudflare Stream module.
 */
class CloudflareStreamSettings extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'cloudflare_stream.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'cloudflare_stream_settings';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    // Getting already saved config from the database.
    $email = $this->config('cloudflare_stream.settings')->get('email');
    $api_key = $this->config('cloudflare_stream.settings')->get('api_key');
    $account_id = $this->config('cloudflare_stream.settings')->get('account_id');
    $zone_id = $this->config('cloudflare_stream.settings')->get('zone_id');
    $replace_source_files = $this->config('cloudflare_stream.settings')->get('replace_source_files');
    $debug_messages = $this->config('cloudflare_stream.settings')->get('debug_messages');

    // Adding form fields to the form.
    $form['email'] = [
      '#type' => 'email',
      '#title' => $this->t('Email Address'),
      '#description' => $this->t('Your Cloudflare login email address'),
      '#default_value' => $email,
      '#required' => TRUE,
    ];
    $form['api_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('API Key'),
      '#description' => $this->t('Your Cloudflare API key. This key can be found on your Cloudflare My Profile page > API tokens tab. <br><a href=":link">More info on the Cloudflare documentation page</a>', [':link' => 'https://support.cloudflare.com/hc/en-us/articles/200167836-Managing-API-Tokens-and-Keys#12345682']),
      '#default_value' => $api_key,
      '#required' => TRUE,
    ];
    $form['domain_specific_settings'] = [
      '#type' => 'details',
      '#open' => TRUE,
      '#title' => $this->t('Domain specific settings'),
      'instruction' => [
        '#markup' => '<p>' . $this->t('The Account & Zone ID can be domain specific. To find these, you go to the Cloudflare Dashboard and select your domain. You find the needed info under the API section in the sidebar.') . "</p>",
      ],
    ];
    $form['domain_specific_settings']['account_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Account ID'),
      '#default_value' => $account_id,
      '#required' => TRUE,
    ];
    $form['domain_specific_settings']['zone_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Zone ID'),
      '#default_value' => $zone_id,
      '#required' => TRUE,
    ];
    $form['replace_source_files'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Do not store the source files locally'),
      '#description' => $this->t("Check this box if you don't want the source files stored on the webserver. The video file will be replaced with an empty file when saved."),
      '#default_value' => $replace_source_files,
    ];
    $form['debug_messages'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Show debug messages'),
      '#description' => $this->t("Check this box if you want to show debug messages in the frontend. By default, we only log the error messages in watchdog."),
      '#default_value' => $debug_messages,
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Trimming user entry for any whitespace.
    $email = preg_replace('/\s/', '', $form_state->getValue('email'));
    $api_key = preg_replace('/\s/', '', $form_state->getValue('api_key'));
    $account_id = preg_replace('/\s/', '', $form_state->getValue('account_id'));
    $zone_id = preg_replace('/\s/', '', $form_state->getValue('zone_id'));
    $replace_source_files = $form_state->getValue('replace_source_files');
    $debug_messages = $form_state->getValue('debug_messages');

    // Saving config in the database.
    $this->config('cloudflare_stream.settings')
      ->set('email', $email)
      ->set('api_key', $api_key)
      ->set('account_id', $account_id)
      ->set('zone_id', $zone_id)
      ->set('replace_source_files', $replace_source_files)
      ->set('debug_messages', $debug_messages)
      ->save();
    $this->messenger()->addMessage($this->t('Cloudflare configurations saved successfully.'));
  }

}
