<?php

namespace Drupal\cloudflare_stream\Plugin\Field\FieldType;

use Drupal\cloudflare_stream\Service\CloudflareStreamApiInterface;
use Drupal\cloudflare_stream\Service\CloudflareStreamInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\TypedData\DataDefinition;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\TypedData\DataDefinitionInterface;
use Drupal\Core\TypedData\TraversableTypedDataInterface;
use Drupal\Core\TypedData\TypedDataInterface;
use Drupal\file\FileUsage\FileUsageInterface;
use Drupal\file\Plugin\Field\FieldType\FileItem;

/**
 * Plugin implementation of the 'cloudflareVideo' field type.
 *
 * @FieldType(
 *   id = "cloudflarevideo",
 *   label = @Translation("Cloudflare Video"),
 *   description = @Translation("This field stores the ID of an cloudflare video."),
 *   category = @Translation("Cloudflare"),
 *   default_widget = "cloudflarevideo_default",
 *   default_formatter = "cloudflarevideo_default",
 *   list_class = "\Drupal\file\Plugin\Field\FieldType\FileFieldItemList",
 *   constraints = {"ReferenceAccess" = {}, "FileValidation" = {}}
 * )
 */
class CloudflareVideoItem extends FileItem {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The file system.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * The Cloudflare Stream service.
   *
   * @var \Drupal\cloudflare_stream\Service\CloudflareStreamInterface
   */
  protected $cloudflareStream;

  /**
   * The Cloudflare Stream API service.
   *
   * @var \Drupal\cloudflare_stream\Service\CloudflareStreamApiInterface
   */
  protected $cloudflareStreamApi;

  /**
   * The file usage service.
   *
   * @var \Drupal\file\FileUsage\FileUsageInterface
   */
  protected $fileUsage;

  /**
   * {@inheritdoc}
   */
  public function __construct(
    DataDefinitionInterface $definition,
    EntityTypeManagerInterface $entity_type_manager,
    FileSystemInterface $file_system,
    CloudflareStreamInterface $cloudflareStream,
    CloudflareStreamApiInterface $cloudflareStreamApi,
    $name = NULL,
    TypedDataInterface $parent = NULL,
    FileUsageInterface $file_usage = NULL
  ) {
    parent::__construct($definition, $name, $parent);
    $this->entityTypeManager = $entity_type_manager;
    $this->fileSystem = $file_system;
    $this->fileUsage = $file_usage;
    $this->cloudflareStream = $cloudflareStream;
    $this->cloudflareStreamApi = $cloudflareStreamApi;
  }

  /**
   * {@inheritdoc}
   */
  public static function createInstance($definition, $name = NULL, TraversableTypedDataInterface $parent = NULL) {
    $entityTypeManager = \Drupal::service('entity_type.manager');
    $fileSystem = \Drupal::service('file_system');
    $fileUsage = \Drupal::service('file.usage');
    $cloudflareStream = \Drupal::service('cloudflare_stream');
    $cloudflareStreamApi = \Drupal::service('cloudflare_stream.api');
    return new static(
      $definition,
      $entityTypeManager,
      $fileSystem,
      $cloudflareStream,
      $cloudflareStreamApi,
      $name,
      $parent,
      $fileUsage
    );
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultFieldSettings() {
    $fieldSettings = parent::defaultFieldSettings();
    $cloudflareStream = \Drupal::service('cloudflare_stream');
    $fieldSettings['file_extensions'] = $cloudflareStream->listAllowedFileExtensions();

    return $fieldSettings;
  }

  /**
   * {@inheritdoc}
   */
  public static function schema(FieldStorageDefinitionInterface $field_definition) {
    $schema = parent::schema($field_definition);
    $schema['columns']['cloudflareStreamVideoID'] = [
      'type' => 'text',
      'length' => 512,
      'not null' => FALSE,
    ];
    $schema['columns']['thumbnail'] = [
      'type' => 'text',
      'length' => 512,
      'not null' => FALSE,
    ];

    return $schema;
  }

  /**
   * {@inheritdoc}
   */
  public function fieldSettingsForm(array $form, FormStateInterface $form_state) {
    $form = parent::fieldSettingsForm($form, $form_state);

    $form['file_extensions']['#description'] = $this->t('Separate extensions with a space or comma and do not include the leading dot.<br>Cloudflare Stream supports following extensions: MP4, MKV, MOV, AVI, FLV, MPEG-2 TS, MPEG-2 PS, MXF, LXF, GXF, 3GP, WebM, MPG, QuickTime');

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition) {
    $properties = parent::propertyDefinitions($field_definition);

    $properties['cloudflareStreamVideoID'] = DataDefinition::create('string')
      ->setLabel(t('Cloudflare Video ID'));
    $properties['thumbnail'] = DataDefinition::create('string')
      ->setLabel(t('Cloudflare Video thumbnail'));
    return $properties;
  }

  /**
   * {@inheritdoc}
   */
  public function preSave() {
    $entity = $this->getEntity();
    if (is_null($this->getValue()['cloudflareStreamVideoID'])) {
      $fileID = $this->get('target_id')->getValue();
      $file = $this->entityTypeManager->getStorage('file')->load($fileID);
      if (isset($file)) {
        $response = $this->cloudflareStreamApi->uploadVideoByHttpRequest($file);

        if (isset($response['result']['uid'])) {
          $videoId = $response['result']['uid'];
          $oldVideoId = $this->getOldVideoId($entity, $this->name);
          if (isset($oldVideoId)) {
            $this->cloudflareStreamApi->deleteVideo($oldVideoId);
          }
          $this->setValue(
            [
              'target_id' => $this->getValue()['target_id'],
              'display' => $this->getValue()['display'],
              'description' => $this->getValue()['description'],
              'cloudflareStreamVideoID' => $videoId,
              'thumbnail' => $response['result']['thumbnail'],
            ]
          );
          // Override file contents, so the original video doesn't get stored on
          // the website.
          if ($this->cloudflareStream->replaceSourceFiles()) {
            file_put_contents($this->fileSystem->realpath($file->getFileUri()), '');
          }
          $this->fileUsage->add($file, 'cloudflare_stream', 'cloudflarevideo', \Drupal::currentUser()->id());
          $file->setPermanent();
          $file->save();
        }
      }
    }
    else {
      $fieldName = $this->getParent()->getName();
      $newFieldData = $this->getEntity()->get($fieldName)->getValue();
      $originalFieldData = $entity->get($fieldName)->getValue();
      $videoIds = array_filter($this->getRemovedIds($originalFieldData, $newFieldData), function ($element) {
        if ($element !== NULL) {
          return $element;
        }
      });

      foreach ($videoIds as $videoId) {
        $this->cloudflareStreamApi->deleteVideo($videoId);
      }
    }
  }

  /**
   * Return the old video ID.
   */
  protected function getOldVideoId($entity, $delta) {
    $oldVideoID = NULL;
    if ($entity) {
      $fieldName = $this->getParent()->getName();
      $oldVideoID = $entity->get($fieldName)->getValue()[$delta]['cloudflareStreamVideoID'] ? $entity->get($fieldName)->getValue()[$delta]['cloudflareStreamVideoID'] : NULL;
    }
    return $oldVideoID;
  }

  /**
   * Get the removed video IDs from cloudflare stream.
   *
   * @param array $originalFieldData
   *   Contains the original video data.
   * @param array $newFieldData
   *   Contains the new video data.
   *
   * @return array
   *   The removed video IDs.
   */
  protected function getRemovedIds(array $originalFieldData, array $newFieldData) {
    $removedIds = [];
    foreach ($originalFieldData as $key => $originalFieldDatum) {
      foreach ($newFieldData as $newFieldDatum) {
        if ($originalFieldDatum['cloudflareStreamVideoID'] === $newFieldDatum['cloudflareStreamVideoID']) {
          $removedIds[$key] = NULL;
          break;
        }
        $removedIds[$key] = $originalFieldDatum['cloudflareStreamVideoID'];
      }
    }

    return $removedIds;
  }

  /**
   * {@inheritdoc}
   */
  public function delete() {
    $this->cloudflareStreamApi->deleteVideo($this->getValue()['cloudflareStreamVideoID']);
  }

}
